# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: install.py 28105 2005-10-10 11:11:33Z lregebro $

from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName, _checkPermission
# Please let this backwards compatibility stay until CPS 3.4.0
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CPSCore.CPSWorkflow import \
     TRANSITION_INITIAL_PUBLISHING, TRANSITION_INITIAL_CREATE, \
     TRANSITION_ALLOWSUB_CREATE, TRANSITION_ALLOWSUB_PUBLISHING, \
     TRANSITION_BEHAVIOR_PUBLISHING, TRANSITION_BEHAVIOR_FREEZE, \
     TRANSITION_BEHAVIOR_DELETE, TRANSITION_BEHAVIOR_MERGE, \
     TRANSITION_ALLOWSUB_CHECKOUT, TRANSITION_INITIAL_CHECKOUT, \
     TRANSITION_BEHAVIOR_CHECKOUT, TRANSITION_ALLOW_CHECKIN, \
     TRANSITION_BEHAVIOR_CHECKIN, TRANSITION_ALLOWSUB_DELETE, \
     TRANSITION_ALLOWSUB_MOVE, TRANSITION_ALLOWSUB_COPY
from Products.DCWorkflow.Transitions import TRIGGER_USER_ACTION
from Products.ExternalMethod.ExternalMethod import ExternalMethod
from Products.CPSInstaller.CPSInstaller import CPSInstaller

from Products.CalZope.permissions import permissions as cal_permissions
from Products.CPSSharedCalendar.calendartool import EVENT_LIST

WebDavLockItem = 'WebDAV Lock items'
WebDavUnlockItem = 'WebDAV Unlock items'

def install(self):
    installer = CPSInstaller(self, 'CPSSharedCalendar')
    
    # CPSSharedCalendar requires CMFonFive
    installer.runExternalUpdater('cmfonfive_installer', 'CMFonFive Installer',
                                 'CMFonFive', 'install', 'install')
    
    # Create tools
    installer.verifyTool('portal_calendar', 'CPSSharedCalendar', 
                         'CPS Calendar Tool')
    installer.verifyTool('portal_cpscalendar', 'CPSSharedCalendar', 
                         'CPS Shared Calendar Tool')
    
    # Make sure the index is up to date:
    self.portal_calendar.storage_manager._storage.reindexAll()
    
    # Set up portal types
    types = {
        'CPS Calendar': {
            'typeinfo_name': 'CPSSharedCalendar: CPS Calendar',
            'add_meta_type': 'Factory-based Type Information',
            }
        }
    installer.verifyContentTypes(types)
    #installer.allowContentTypes('CPS Calendar', 'Workspace')
    # Remove the old calendar
    ttool = installer.getTool('portal_types')
    for type in ttool.objectIds():
        workspaceACT = list(ttool[type].allowed_content_types)
        if 'Calendar' in  workspaceACT or type == 'Workspace':
            if 'Calendar' in  workspaceACT:
                workspaceACT.remove('Calendar')
            if 'CPS Calendar' not in workspaceACT:
                workspaceACT.append('CPS Calendar')
            ttool[type].allowed_content_types = workspaceACT

    # Setup workflow workspace_calendar_wf
    wfdef = {'wfid': 'workspace_calendar_wf',
             'permissions': (View, ModifyPortalContent,
                             WebDavLockItem, WebDavUnlockItem,),
             'state_var': 'review_state',
             }

    wfstates = {
        'work': {
            'title': 'Work',
            'transitions': ('copy_submit', 'checkout_draft',
                            'cut_copy_paste', 'modify'),
            'permissions': {View: ('Manager', 'WorkspaceManager',
                                   'WorkspaceMember', 'WorkspaceReader'),
                            ModifyPortalContent: ('Manager', 'Owner',
                                                  'WorkspaceManager', 'WorkspaceMember'),
                            WebDavLockItem: ('Manager', 'Owner',
                                             'WorkspaceManager', 'WorkspaceMember'),
                            WebDavUnlockItem: ('Manager', 'Owner',
                                               'WorkspaceManager', 'WorkspaceMember')},
            },
        }
    
    wftransitions = {
        'create': {
            'title': 'Initial creation',
            'new_state_id': 'work',
            'transition_behavior': (TRANSITION_INITIAL_CREATE, ),
            'clone_allowed_transitions': None,
            'actbox_name': '',
            'actbox_category': 'workflow',
            'actbox_url': '',
            'props': {'guard_permissions': '',
                      'guard_roles': 'Manager; WorkspaceManager; '
                      'WorkspaceMember',
                      'guard_expr': ''},
            'after_script_name' : 'give_attendee_manager',
            },
        'cut_copy_paste': {
            'title': 'Cut/Copy/Paste',
            'new_state_id': '',
            'transition_behavior': (TRANSITION_ALLOWSUB_DELETE,
                                    TRANSITION_ALLOWSUB_MOVE,
                                    TRANSITION_ALLOWSUB_COPY),
            'clone_allowed_transitions': None,
            'trigger_type': TRIGGER_USER_ACTION,
            'actbox_name': '',
            'actbox_category': '',
            'actbox_url': '',
            'props': {'guard_permissions': '',
                      'guard_roles': 'Manager; WorkspaceManager; '
                      'WorkspaceMember',
                      'guard_expr': ''},
            },
        'modify': {
            'title': 'Modification of content,'
            'provides a specific entry in status history',
            'new_state_id': '',
            'transition_behavior': (),
            'clone_allowed_transitions': None,
            'trigger_type': TRIGGER_USER_ACTION,
            'actbox_category': 'workflow',
            'props': {'guard_permissions': '',
                      'guard_roles': 'Manager; Owner;'
                      'WorkspaceManager; '
                      'WorkspaceMember',
                      'guard_expr': ''},
            },
        }
    
    wfvariables = {
        'action': {
            'description': 'The last transition',
            'default_expr': 'transition/getId|nothing',
            'for_status': 1,
            'update_always': 1,
            },
        'actor': {
            'description': 'The ID of the user who performed',
            'default_expr': 'user/getId',
            'for_status': 1,
            'update_always': 1,
            },
        'comments': {
            'description': 'Comments about the last transition',
            'default_expr': "python:state_change.kwargs.get('comment', '')",
            'for_status': 1,
            'update_always': 1,
            },
        'time': {
            'description': 'Time of the last transition',
            'default_expr': 'state_change/getDateTime',
            'for_status': 1,
            'update_always': 1,
            'for_catalog': 1,
            },
        'dest_container': {
            'description': 'Destination container for the last '
            'paste/publish',
            'default_expr': "python:state_change.kwargs.get("
            "'dest_container', '')",
            'for_status': 1,
            'update_always': 1,
            },
        }

    ### give_attendeee_manageer: to give the creator the AttendeeManager
    ## role, which has a special meaning (see ZODBEvent class) 
    wfscripts = {
        'give_attendee_manager' : {
           '_owner' : None,
           '_proxy_roles' : ['Manager'],
           'script' : """\
##parameters=state_change           
context = state_change.object 
from Products.CMFCore.utils import getToolByName       
mtool = getToolByName(context, 'portal_membership')
user_id = mtool.getAuthenticatedMember().getUser().getId()
mtool.setLocalRoles(context, [user_id],
                        'AttendeeManager', reindex=1)
"""
           }
        }

    installer.verifyWorkflow(wfdef, wfstates, wftransitions,
                        wfscripts, wfvariables)

    installer.verifyLocalWorkflowChains(
        installer.portal['workspaces'],
        {'CPS Calendar': 'workspace_calendar_wf'}
        )
    
    # Set up permissions and roles:
    roles = ('EventOrganizer', 'EventParticipant', 'EventReader',
             'AttendeeManager', 'AttendeeReader')
    if getattr(self.acl_users, 'userFolderAddRole', None) is None:
        verifyRoles(installer, roles)
    else:
        installer.verifyRoles(roles)
    
    installer.setupPortalPermissions(cal_permissions, installer.portal)
    
    # Setup subscriptions
    stool = installer.getTool('portal_subscriptions')
    for event in EVENT_LIST:
        event_id = 'calendar_event_%s' % event
        stool.manage_addEventType('CPS Calendar', event_id, event_id)
        stool.manage_addEventType('Workspace', event_id, event_id)
        stool.manage_addEventType('Section', event_id, event_id)
        
    stool.manage_addPortalTypeToArea('Workspace', 'CPS Calendar')
    
    for role in roles + ('Owner',):
        stool._p_changed = 1
        stool.manage_addLocalRoleToPortalTypeToArea('Workspace', 'CPS Calendar',
                                              role, role)
        #stool.manage_addLocalRoleToPortalTypeToArea('Workspace', 'Workspace',
                                              #role, role)
        #stool.manage_addLocalRoleToPortalTypeToArea('Section', 'Section',
                                              #role, role)

    rolle = {}
    for role, title in stool.getLocalRoleArea('Workspace')['Workspace'].items():
        if role in roles:
            continue
        rolle[role] = title
    stool.mapping_local_roles_context['Workspace']['Workspace'] = rolle
    stool._p_changed = 1
        

    # Set up better messages
    stool.manage_editEventMessage('calendar_event_invite',
        event_email_title='[Calendar] %(user_name)s has invited you to a meeting',
        event_email_body='%(user_name)s has invited you to a meeting:\n'
            'Title: %(event_title)s\n'
            'Starts: %(event_begins)s\n'
            'Ends: %(event_ends)s\n\n'
            'You can accept or deny the meeting invitation at \n'
            '%(event_url)s/attendees.html\n\n'
            'For further event details, please go to \n'
            '%(event_url)s .\n'
        )

    stool.manage_editEventMessage('calendar_event_status_change',
        event_email_title='[Calendar] Your attendence status has been changed',
        event_email_body='%(user_name)s has changed your attendence status for '
            'an event.\n'
            'Title: %(event_title)s\n'
            'Starts: %(event_begins)s\n'
            'Ends: %(event_ends)s\n\n'
            'Your status was changed from %(old_status)s to %(new_status)s\n'
            'You can change the attendence status at \n'
            '%(event_url)s/attendees.html\n\n'
            'For further event details, please go to \n'
            '%(event_url)s\n'
        )
                                 
    stool.manage_editEventMessage('calendar_event_modify',
        event_email_title='[Calendar] Event modification',
        event_email_body='A meeting or event that you are attending has been '
            'moved or updated:\n\n'
            'Title: %(event_title)s\n'
            'Starts: %(event_begins)s\n'
            'Ends: %(event_ends)s\n\n'
            'For further event details, please go to \n'
            '%(event_url)s\n'
        )

    stool.manage_editEventMessage('calendar_event_deleted',
        event_email_title='[Calendar] Event deleted',
        event_email_body='%(user_name)s has deleted a meeting or event that '
            'you were to attend:\n'
            'Title: %(event_title)s\n'
            'Starts: %(event_begins)s\n'
            'Ends: %(event_ends)s\n\n'
        )

    # Set up default subscriptions for Owner and AttendeeManagers
    membersfolder = self.workspaces.members
    subs = stool.getSubscriptionContainerFromContext(membersfolder)
    if subs is None:
        # Here, no subscriptions container has been created.
        # Force the creation localy
        subs = stool.getSubscriptionContainerFromContext(
            membersfolder, force_local_creation=True)
    for subid in EVENT_LIST:
        if getattr(subs, 'getSubscriptionById', None) is None:
            sub = getSubscriptionById(subs, 'calendar_event_%s' % subid)
        else:
            sub = subs.getSubscriptionById('calendar_event_%s' % subid)
        rule = sub.getRecipientsRules("Role Recipient Rule")
        if len(rule) > 0:
            rule = rule[0]
        else:
            sub.manage_addProduct['CPSSubscriptions'].addRoleRecipientsRule()
            rule = sub.getRecipientsRules("Role Recipient Rule")[0]
        roles = rule.getRoles()
        if 'AttendeeManager' not in roles:
            rule.addRole('AttendeeManager')

    installer.verifyAction(
        'portal_actions',
        id='my_calendar',
        name='My calendar',
        action='string:${portal/portal_cpscalendar/getHomeCalendarUrl}',
        condition="portal/portal_cpscalendar/getHomeCalendarObject",
        permission=('View',),
        category='user',
        visible=1)
    installer.verifyAction(
        'portal_actions',
        id='create_my_calendar',
        name='Create my home calendar',
        action='string:${portal_url}/portal_cpscalendar/createMemberCalendar',
        condition="python:portal.portal_membership.getHomeFolder() and \
                   not portal.portal_cpscalendar.getHomeCalendarObject() and \
                   getattr(portal.portal_cpscalendar, \
                   'member_can_create_home_calendar', 1)",
        permission=('View',),
        category='user',
        visible=1)

    #installer.verifyMessageCatalog('calendar', 'Calendaring')
    installer.setupTranslations(product_name="CalZope", message_catalog="default")
    installer.setupTranslations(product_name="CPSSharedCalendar", message_catalog="default")
    
    
    #Set up permissions
    from Products.CalZope.permissions import permissions
    for permission, roles in permissions.items():
        pms = self.rolesOfPermission(permission)
        p_roles = [r['name'] for r in pms if r['selected']]
        for role in roles:
            if role not in p_roles:
                p_roles.append(role)
            # Add some default CPS roles to the permissions:
            if 'AttendeeReader' in p_roles and 'WorkspaceReader' not in p_roles:
                p_roles.extend(['WorkspaceManager', 'WorkspaceMember', 'WorkspaceReader'])
            if 'AttendeeManager' in p_roles and 'WorkspaceManager' not in p_roles:
                p_roles.append('WorkspaceManager')
        self.manage_permission(permission, p_roles, 1)

    script = ExternalMethod('cpssharedcalendar_upgrade', 
                            'CPSSharedCalendar Upgrade', 
                            'CPSSharedCalendar.upgrade',
                            'upgrade_100_120')
    script = script.__of__(self)
    result = script()
    installer.log(result)

    return installer.flush()


# Helper methods when the site has become unstable because of upgrades from
# alpha-versions.
def recreate_calendar(obj, path):
    # Currently only recreates member calendars.
    parent = obj.aq_parent
    id = obj.id
    
    type = getattr(obj, '_attendee_type', 'INDIVIDUAL')
         
    if type == 'INDIVIDUAL':
        name = getattr(obj, '_attendee_name', None)
        if name == None:
            name = obj.getOwnerId()
            cal = parent.portal_cpscalendar.getHomeCalendarUrl(name)
            if cal != path:
                # This is not a member calendar
                return
        parent._delObject(id)
        parent.portal_cpscalendar.createMemberCalendar(name)

def recreate_calendars(self):
    # This is non-destructive, but costly.
    self.ZopeFindAndApply(self, obj_metatypes='CPS Calendar', 
                                 search_sub=1, apply_func=recreate_calendar)
    return "Calendars recreated sucessfully!"
    
# Support methods for really early CPS versions ( < 3.2.4):

def verifyRoles(self, roles):
    already = self.portal.valid_roles()
    for role in roles:
        if role not in already:
            self.portal._addRole(role)
            self.log(" Add role %s" % role)

def getSubscriptionById(self, subscription_id=''):
    """Return a susbcription object given an id
    """
    subtool = getToolByName(self, 'portal_subscriptions')
    subscription_prefix = 'subscription__'

    if not subscription_id.startswith(subscription_prefix):
        subscription_id = subscription_prefix + subscription_id

    subscription =  getattr(self, subscription_id, None)

    if (subscription is None and
        _checkPermission(ModifyPortalContent, self)):
        container = self.addSubscription(subscription_id)
        subscription =  getattr(self, subscription_id, None)

    return subscription


