# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: install.py 26143 2005-08-26 14:38:34Z lregebro $

import transaction
from Products.ZCatalog.ZCatalog import ZCatalog
from Products.CPSSharedCalendar.calendar import manage_addCpsCalendar
from Products.CPSSharedCalendar.calendartool import EVENT_LIST
from DateTime import DateTime    
    
def _reindexAll(self):
    # Updates the event indexing:
    self.portal_calendar.storage_manager._storage.reindexAll()

def upgrade_100_120(self):
    # Upgrades to CPSSharedCalendars 1.0.0 and later to 1.2.0.
    result = '<h3>Upgrade to 1.2.0: </h3>'

    # Users should in 1.2.0 have AttendeeManager on their personal calendars 
    # to ensure correct notifications.
    # 1. Owner should no longer be notified!
    stool = self.portal_subscriptions
    membersfolder = self.workspaces.members
    subs = stool.getSubscriptionContainerFromContext(membersfolder)
    empty_count = 0
    for subid in EVENT_LIST:
        sub = subs.getSubscriptionById('calendar_event_%s' % subid)
        rules = sub.getRecipientsRules("Role Recipient Rule")
        # A bug in the installer ment that an empty rule would be 
        # created on  each install. Remove all empty rules:
        for rule in rules:
            if rule.roles == []:
                sub._delObject(rule.getId())
                empty_count += 1
        
        # Get the remaining rules:
        rules = sub.getRecipientsRules("Role Recipient Rule")
        if len(rules) > 0:
            roles = rules[0].getRoles()
            # Owner is no longer default:
            if 'Owner' in roles:
                roles.remove('Owner')
                rules[0].roles = roles
                result += '<p>Upgraded notification settings for ' \
                          'calendar_event_%s events.</p>' % subid

    if empty_count:
        result += '<p>Removed %s empty subscription rules.</p>' % empty_count
    
    # 2. Everybody should have 'AttendeeManager' on themselves.
    attsrc = self.portal_calendar.attendee_source
    
    attendees = attsrc.findByName('*', 'INDIVIDUAL')
    for attendee in attendees:
        member_id = attendee.getUserId()
        cal = self.portal_cpscalendar.getHomeCalendarObject(member_id)
        roles = cal.get_local_roles_for_userid(member_id)
        if not 'AttendeeManager' in roles:
            roles = roles + ('AttendeeManager',)
            cal.manage_setLocalRoles(member_id, roles)
            result += '<p>Updated local roles for member calendar %s.</p>'  % member_id

    # 3. The titles of the roles should be the role id so they get translated.
    changed = 0
    for role in ('EventOrganizer', 'EventParticipant', 'EventReader',
             'AttendeeManager', 'AttendeeReader'):
        if not stool.getLocalRoleArea('Workspace')['CPS Calendar'].get(role, role) == role:
            stool.getLocalRoleArea('Workspace')['CPS Calendar'][role] = role
            changed = 1
        if not stool.getLocalRoleArea('Workspace')['Workspace'].get(role, role) == role:
            stool.getLocalRoleArea('Workspace')['Workspace'][role] = role
            changed = 1
        if not stool.getLocalRoleArea('Section')['Section'].get(role, role) == role:
            stool.getLocalRoleArea('Section')['Section'][role] = role
            changed = 1
    if changed:
        result += '<p>Set correct labels for subscription roles.</p>'

    stool.manage_addLocalRoleToPortalTypeToArea('Workspace', 'CPS Calendar',
                                                'Owner', 'label_owner')
    
    # 4. Create the indexes
    if getattr(self.portal_calendar.storage_manager._storage, 
               '_indexes', None) is None:
        _reindexAll(self)
        result += '<p>Upgraded event index and reindexed all events.</p>'
        
    result += '<p><b>Upgrade done.</b></p>'
    return result

def upgrade(self):
    # Try to make a clever upgrade
    
    # Make sure the new calendar has been installed (must be done first)
    caltool = self.portal_calendar
    sm = getattr(caltool, 'storage_manager', None)
    result = '<html><body><p>Upgrading CPS Shared Calendar...</p>'
    if sm is None:
        result += "<p>CPS Shared Calendar is not installed!</p></body></html>"
        return

    # Check if indexes exist. If they do, assume version 1.2.0 or later:
    if getattr(sm._storage, '_indexes', None) is None:
        result += upgrade_100_120(self)

    result += '<p>No further upgrades needed.</p></body></html>'
    return result


# Migration support:

def listOldCalendars(self):
    """Return all available CPSCalendar objects in a list"""
    brains = ZCatalog.searchResults(self.portal_catalog,
             meta_type='Calendar')
    if brains:
        # This is an unrestricted version of catalogbrains getObject():
        calendars = [brain.aq_parent.unrestrictedTraverse(brain.getPath())
                     for brain in brains]
        # Filter any empty entries.
        return [each for each in calendars if each] 
    else:
        return []

def migrate_from_cpscalendar(self):
    caltool = self.portal_cpscalendar
    if caltool.meta_type != 'CPS Shared Calendar Tool':
        raise "You must run the installer for CPS Shared Calendar Tool first!"

    # Find all calendars. We do this by a Catalog search,
    # In the odd event that some calendars are not properly indexed, these
    # will the not be upgraded, but that's easy to fix by indexing them and
    # running this method again.
    
    calendars = listOldCalendars(self)

    for calendar in calendars:
        icaldata = oldCalendarExport(calendar)

        parent = calendar.aq_parent
        id = calendar.getId()
        caltype = calendar.usertype
        parent.manage_delObjects([id])
        if  caltype == 'member':
            member = calendar.getOwnerId()
            newcal = caltool.createMemberCalendar(member)
        else:
            if caltype in ('resource', 'room'):
                caltype = 'RESOURCE'
            else:
                caltype = 'WORKSPACE'
            wtool.invokeFactoryFor(parent, 'CPS Calendar', id,
                               title=calendar.title,
                               attendee_type=caltype)
            newcal = parent._getOb(id)
        
        newcal.import_(icaldata)
        transaction.commit()

    return "CPSCalendars successfully migrated."

def deleteOldCalendars(self):

    # Delete all CPSCalendar calendars. 
    brains = ZCatalog.searchResults(self.portal_catalog,
             meta_type='Calendar')
    if not brains:
        return
    
    for brain in brains:
        calendar = brain.aq_parent.unrestrictedTraverse(brain.getPath())
        if calendar:
            parent = calendar.aq_parent
            id = calendar.getId()
            parent.manage_delObjects([id])
            transaction.commit()

    return "All CPSCalendars deleted."

def icalvalue(s):
    return unicode('\\,'.join(s.split(',')), 'latin1').encode('UTF-8')

def attendee2ical(event, attendee):
    cutype = attendee.get('usertype', 'member').upper()
    if  cutype == 'MEMBER': 
        cid = attendee['id']
        mail = event.getMemberEmail(cid)
        message = '' # People is the default CUTYPE (INDIVIDUAL).
    else:
        if cutype not in ('GROUP', 'RESOURCE', 'ROOM'):
            # Mostly to handle the special "Workspace" type.
            cutype = 'X-' + cutype
        message = 'CUTYPE=%s;' % cutype
        cid = attendee['rpath']
        mail = 'nomail@nohost.no'

    status = attendee.get('status')
    if status == 'unconfirmed':
        status = 'NEEDS-ACTION'
    elif status == 'confirmed':
        status = 'ACCEPTED'
    elif status == 'tentative':
        status = 'TENTATIVE'
    elif status == 'decline':
        status = 'DECLINED'
    else:
        status = None
    if status is not None:
        message += 'PARTSTAT=%s;' % status
        
    # iCalendar identifies calendars by email adress, which is nonsensical.
    # We therefore add X-CID (Calendar ID) which contains the id, so that
    # we at least understand what we are talking about.
    message += 'X-CID=%s:MAILTO:%s\n' % (cid, mail)
    
    return message

def oldCalendarExport(self):
    ical_conv = '%Y%m%dT%H%M%S'
    ical_date_conv = '%Y%m%d'

    header = """BEGIN:VCALENDAR
CALSCALE:GREGORIAN
X-WR-TIMEZONE;VALUE=TEXT:Europe/Paris
PRODID:-//Nuxeo//CPSCalendar 0.1//EN
X-WR-CALNAME;VALUE=TEXT:%s
X-WR-RELCALID;VALUE=TEXT:%s
VERSION:2.0
""" % (icalvalue(self.title_or_id()), self.absolute_url())

    footer = """\
END:VCALENDAR
"""

    message = header

    events = self.objectValues('Event')
    dtstamp = DateTime('UTC')
    
    event_header = """\
BEGIN:VEVENT
DTSTAMP:%s
""" % (dtstamp.strftime(ical_conv), )

    event_footer = """\
END:VEVENT
"""

    for event in events:
        eventfrom = event.from_date.toZone('UTC').strftime(ical_conv)
        eventto = event.to_date.toZone('UTC').strftime(ical_conv)
        
        message += event_header
        
        message += 'SUMMARY:%s\n' % icalvalue(event.title_or_id())
        message += 'UID:%s\n' % event.getId()
        message += 'DESCRIPTION:%s\n' % event.description
        if event.location:
            message += 'LOCATION:%s\n'% icalvalue(event.location)
        if event.document:
            message += 'ATTACH:%s\n'% icalvalue(event.document)
        # The missspelling 'canceled' is used for cancelled status...
        message += 'STATUS:%s\n' % ('canceled' and 'CANCELLED' or event.status.upper())
        # This defaults to OPAQUE, so we only include it when not:
        if event.transparent:
            message += 'TRANSP:TRANSPARENT\n'
        # This defaults to PUBLIC
        if event.category == 'private':
            message += 'CLASS:PRIVATE\n'
        
        if event.event_type == 'event_allday':
            message += 'DTSTART;VALUE=DATE:%s\n' % event.from_date.strftime(ical_date_conv)
            message += 'DTEND;VALUE=DATE:%s\n' % (event.to_date+1).strftime(ical_date_conv)
        else:
            message += 'DTSTART:%s\n' % eventfrom
            message += 'DTEND:%s\n' % eventto

        # Organizer (Yes, it should be a semicolon):
        if event.organizer:
            message += 'ORGANIZER;' + attendee2ical(event, event.organizer)
    
        for each in event.attendees:
            message += 'ATTENDEE;' + attendee2ical(event, each)

        if event.event_type == 'event_recurring':
            message += 'RRULE:FREQ='
            if event.recurrence_period == 'period_daily':
                message += 'DAILY'
            elif event.recurrence_period == 'period_weekly':
                message += 'WEEKLY'
            elif event.recurrence_period == 'period_monthly':
                message += 'MONTHLY'
            elif event.recurrence_period == 'period_quarterly':
                message += 'MONTHLY;INTERVAL=3'
            elif event.recurrence_period == 'period_yearly':
                message += 'YEARLY'
            else:
                raise 'Unknown recurrance period', event.recurrence_period
            message += '\n'
            
        message += event_footer
        
    message += footer
    return message
    