# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: calendar.py 27868 2005-10-03 14:30:13Z lregebro $

from zope.interface import implements

from OFS.PropertyManager import PropertyManager

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CPSCore.CPSBase import CPSBaseDocument
from Products.CPSonFive.form.interfaces import IDocumentNavigation
from Products.CPSNavigation.CPSNavigation import CPSNavigation

from Products.CalZope.zopecal import Calendar, EventList, WeekList, Year
from Products.Five.traversable import  FiveTraversable
from calendartool import CpsAttendee

factory_type_information = {
     'id': 'CPS Calendar',
     'title': 'portal_type_Calendar_title',
     'description': 'portal_type_Calendar_description',
     'icon': '++resource++cpssharedcalendar.png',
     'product': 'CPSSharedCalendar',
     'meta_type': 'CPS Calendar',
     'factory': 'manage_addCpsCalendar',
     'immediate_view': 'calendar.html',
     'filter_content_types': 0,
     'allowed_content_types': (),
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'string:${object_url}/calendar.html',
                  'permissions': ('View',),
                  'visible': 0,
                  },)
     }

from zope.i18nmessageid import MessageIDFactory
_ = MessageIDFactory("Default")

class CpsCalendar(CMFCatalogAware, PropertyManager, Calendar, PortalContent, 
                  DefaultDublinCoreImpl):
    
    meta_type = 'CPS Calendar'
    _attendee_type = 'INDIVIDUAL' # For backwards compatibility
    
    implements(IDocumentNavigation)
    
    def __init__(self, id, title='', attendee_name='', 
                 attendee_type='WORKSPACE'):
        Calendar.__init__(self, id, title)
        self._attendee_name = attendee_name
        self._attendee_type = attendee_type

    def getMainAttendee(self):
        caltool = getToolByName(self, 'portal_calendar').attendee_source
        if self._attendee_type == 'INDIVIDUAL':
            return caltool.getAttendee('INDIVIDUAL!' + self._attendee_name)

        path = '/'.join(self.getPhysicalPath())
        attendee_id = '!'.join(('CALENDAR', path))
        return caltool.getAttendee(attendee_id)

    def getAttendees(self):
        return [self.getMainAttendee()] + Calendar.getAttendees(self)

    def title_or_id(self):
        # XXX Switch to _() syntax when we start using Five 1.1/ Zope3 i18n.
        localizer = getToolByName(self, 'Localizer')
        _ = localizer.default
        if self._attendee_type == 'INDIVIDUAL':
            dict = {'title': self.getMainAttendee().title.decode('ISO-8859-15')}
            str = _("Personal calendar for %(title)s") % dict
        else:
            parent_title = self.aq_inner.aq_parent.title_or_id()
            if not isinstance(parent_title, unicode):
                parent_title = parent_title.decode('ISO-8859-15')
            dict = {'title': self.title.decode('ISO-8859-15'),
                    'parent': parent_title}
            if self._attendee_type == 'WORKSPACE':
                str = _("Workspace calendar %(title)s for %(parent)s") % dict
            elif self._attendee_type == 'RESOURCE':
                str = _("Resource calendar %(title)s for %(parent)s") % dict
            else:
                raise ValueError, "Unkown calendar type %s" % self._attendee_type
        return str
    
    def edit(self, title):
        """CPS doesn't pass the title to the constructor.

        It does call edit() with the title however, so this fixes the problem."""
        self.title = title

    def getCalendarNavigation(self, finder, root_uid, current_uid=None, REQUEST=None):
        if REQUEST and REQUEST.has_key('current_uid'):
            current_uid = REQUEST.get('current_uid')
        if not current_uid:
            current_uid = root_uid
        
        nav = CPSNavigation(root_uid=root_uid,
                            current_uid=current_uid,
                            context=self,
                            include_root=0,
                            no_leaves=0,
                            sort_listing_by='title',
                            sort_listing_direction='asc',
                            batch_size=15,
                            request_form=REQUEST.form,
                            )
        tree = None
        if not REQUEST.get('search'):
            tree = nav.getTree()
        else:
            tree = None
        listing, listing_info, batch_info = nav.getListing()
        return tree, listing, batch_info, listing_info, finder

class CpsCalendarTraversable(FiveTraversable):

    # url space

    # day: calendar/2005/05/30
    # month: calendar/2005/05
    # year: calendar/2005
    # week: calendar/week/2005/3
    # weekday: calendar/week/2005/3/2
    # event: calendar/event/1223

    def traverse(self, name, furtherPath):
        if name == 'event':
            return CpsEventList().__of__(self._subject)
        elif name == 'week':
            return CpsWeekList().__of__(self._subject)
        else:
            try:
                year = int(name)
                return Year(year).__of__(self._subject)
            except ValueError:
                pass
        if name == 'getId':
            # XXX Special hack I don't understand.
            # Maybe I can get rid of it now when I use Traversal adapters?
            return getattr(self._subject, name)
        
        return FiveTraversable.traverse(self, name, furtherPath)


class CpsEventList(EventList):
    """I18N version of event list

    This class can be removed when we start using pure Zope3 i18n.
    """

    def title_or_id(self):
        """Return a translated title for the breadcrumb"""
        # XXX Switch to _() syntax when we start using Five 1.1/ Zope3 i18n.
        localizer = getToolByName(self, 'Localizer')
        _ = localizer.default
        return _(self.title)


class CpsWeekList(WeekList):
    """I18N version of event list

    This class can be removed when we start using pure Zope3 i18n.
    """

    def title_or_id(self):
        """Return a translated title for the breadcrumb"""
        # XXX Switch to _() syntax when we start using Five 1.1/ Zope3 i18n.
        localizer = getToolByName(self, 'Localizer')
        _ = localizer.default
        return _(self.title)

def manage_addCpsCalendar(context, id, title='', attendee_name='', 
                          attendee_type='WORKSPACE', REQUEST=None):
    """Add a calendar."""
    obj = CpsCalendar(id, title, attendee_name, attendee_type)
    context._setObject(id, obj)
    if REQUEST is not None:
        REQUEST.response.redirect('edit.html')
