# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: calendartest.py 27799 2005-10-01 13:42:35Z lregebro $
"""
  CPSSharedCalendar Functional Tests
"""

import unittest, traceback, base64, sys
from datetime import datetime, timedelta
import zLOG

import transaction
from StringIO import StringIO
from ZPublisher.Response import Response
from ZPublisher.Request import Request
from ZPublisher.Test import publish_module

from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import setSecurityManager

from Testing.ZopeTestCase import FunctionalTestCase, installProduct
from Testing.ZopeTestCase import user_name, folder_name
from Testing.ZopeTestCase.functional import ResponseWrapper

from Products.CPSInstaller.CPSInstaller import CPSInstaller
from Products.CPSDefault.tests.CPSDefaultTestCase import CPSDefaultTestCase

installProduct('CalCore')
installProduct('CalZope')
installProduct('CMFonFive')
installProduct('Five')
installProduct('CPSSharedCalendar')

from Products.Five.zcml import load_config
import Products.CalZope
load_config('configure.zcml', package=Products.CalZope)
load_config('overrides.zcml', package=Products.CalZope)
import Products.CMFonFive
load_config('configure.zcml', package=Products.CMFonFive)
import Products.CPSonFive
load_config('configure.zcml', package=Products.CPSonFive)
load_config('overrides.zcml', package=Products.CPSonFive)
import Products.CPSSharedCalendar
load_config('configure.zcml', package=Products.CPSSharedCalendar)

manager_id = 'manager'

class CalendarTestCase(CPSDefaultTestCase, FunctionalTestCase):

    # Some useful constants
    add_product_path = 'manage_addProduct/CPSSharedCalendar'
    tool_name = 'portal_calendar'
    cpstool_name = 'portal_cpscalendar'
    calendar_name = 'calendar'
    event_name = 'event'

    def getTraceback(self, response):
        return getattr(response._response, '_text_traceback', None)

    def assertResponse(self, response, status, message=''):
        if not response.getStatus() == status:
            tb = self.getTraceback(response)
            if tb is not None:
                raise self.failureException, tb
            else:
                raise ValueError, "Response had status %s, expected %s. %s" % (
                    response.getStatus(), status, message)

    def afterSetUp(self):
        self.login(manager_id)
        installer = CPSInstaller(self.portal, 'CPSSharedCalendar')
        installer.runExternalUpdater('cpssharedcalendar_installer', '', 
                                     'CPSSharedCalendar', 'install', 'install')

        # Set up the home folders and calendars
        # First delete the one that is already there
        for id in self.portal.workspaces.members.objectIds():
            if id[0] != '.':
                self.portal.workspaces.members._delObject(id)
        mtool = self.portal.portal_membership
        mtool.createMemberArea(manager_id)
        mtool.createMemberArea(user_name)

    def print_log_errors(self, min_severity=zLOG.INFO):
        if hasattr(zLOG, 'old_log_write'):
            return
        def log_write(subsystem, severity, summary, detail, error,
                      PROBLEM=zLOG.PROBLEM, min_severity=min_severity):
            if severity >= min_severity:
                print "%s(%s): %s, %s" % (subsystem, severity, summary, detail)
        zLOG.old_log_write = zLOG.log_write
        zLOG.log_write = log_write
    
    def ignore_log_errors(self):
        if hasattr(zLOG, 'old_log_write'):
            zLOG.log_write = zLOG.old_log_write
            del zLOG.old_log_write
    
    def publish(self, path, basic=None, env=None, extra=None, request_method='GET', stdin=None):
        try:
            import Zope2
            return self.publish28(path, basic, env, extra, request_method, stdin)
        except ImportError:
            #Zope 2.7
            return FunctionalTestCase.publish(self, path, basic, env, extra, request_method, stdin)
        
    def publish28(self, path, basic=None, env=None, extra=None, request_method='GET', stdin=None):
        '''Publishes the object at 'path' returning a response object.'''

        # Save current security manager
        sm = getSecurityManager()

        # Commit the sandbox for good measure
        transaction.commit()

        if env is None:
            env = {}
        if extra is None:
            extra = {}

        request = self.app.REQUEST

        env['SERVER_NAME'] = request['SERVER_NAME']
        env['SERVER_PORT'] = request['SERVER_PORT']
        env['REQUEST_METHOD'] = request_method

        p = path.split('?')
        if len(p) == 1:
            env['PATH_INFO'] = p[0]
        elif len(p) == 2:
            [env['PATH_INFO'], env['QUERY_STRING']] = p
        else:
            raise TypeError, ''

        if basic:
            env['HTTP_AUTHORIZATION'] = "Basic %s" % base64.encodestring(basic)

        if stdin is None:
            stdin = StringIO()

        outstream = StringIO()
        response = Response(stdout=outstream, stderr=sys.stderr)

        # In Zope 2.8 these seems to have to sit directly on the request or
        # Five will not find them.
        request=Request(stdin, env, response)
        for k,v in extra.items():
            setattr(request, k, v)

        publish_module('Zope2', response=response, stdin=stdin, environ=env, request=request)

        # Restore security manager
        setSecurityManager(sm)

        return ResponseWrapper(response, outstream, path)

