# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: permissions.py 23787 2005-06-10 13:40:44Z lregebro $
"""
  CPSSharedCalendar Functional Tests
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))
   
import unittest
from datetime import datetime, timedelta
from calendartest import CalendarTestCase
from Testing.ZopeTestCase import user_name
from Products.CPSSharedCalendar.tests.test_notification import DummyMailHost
manager_id = 'manager'

try:
    from Products.CPSSubscriptions.EventManager import get_event_manager
except ImportError:
    # Using CPS < 3.3
    def dummy_event_manager():
        pass
    def get_event_manager():
        return dummy_event_manager
    
class testNotification(CalendarTestCase):

    def afterSetUp(self):
        CalendarTestCase.afterSetUp(self)

        # Make sure the users have emails
        mtool = self.portal.portal_directories.members
        entry = mtool.getEntry('test_user_1_')
        entry['email'] = 'test_user_1_@bar.no'
        mtool.editEntry(entry)
        entry = mtool.getEntry('manager')
        entry['email'] = 'manager@bar.no'
        mtool.editEntry(entry)

        # Set up the dummy mailhost
        self.portal._delObject('MailHost')
        self.portal._setObject('MailHost', DummyMailHost())


    def test_eventEdit(self):
        caltool = self.portal.portal_calendar
        mailhost = self.portal.MailHost
        # Notifictions will not be sent to whomever is logged in, so make sure
        # we know who is logged in:
        self.login('test_user_1_')

        # Create an event and invite someone:
        person1 = caltool.attendee_source.getAttendee('INDIVIDUAL!test_user_1_')
        person2 = caltool.attendee_source.getAttendee('INDIVIDUAL!manager')
        event = person1.createEvent(dtstart=datetime(2005, 2, 10, 14, 00),
                                    duration=timedelta(minutes=60),
                                    title='An event')
        event.invite([person2])
        event.setParticipationStatus(person2, 'ACCEPTED')

        # Clear the mail log:
        em = get_event_manager()
        em._events={}
        mailhost.mail_log = []

        mtool = self.portal.portal_membership
        # And edit the event
        response = self.publish(
            '/portal/workspaces/members/%s/calendar/event/%s/edit.html' % (
                mtool.getHomeFolderId('test_user_1_'), event.getId()),
            basic='%s:secret' % user_name,
            extra={'form': {'field.dtstart': '2005-04-01',
                            'field.dtstart_hour': '13',
                            'field.dtstart_minute': '00',
                            'field.dtend': '2005-04-01',
                            'field.dtend_hour': '15',
                            'field.dtend_minute': '30',
                            'field.title': 'New title!',
                            'field.status': 'TENTATIVE',
                            'field.access': 'PUBLIC',
                            'UPDATE_SUBMIT': 'Edit'}})
        self.assertResponse(response, 200)
        # Check the notification:
        self.failIfEqual(len(mailhost.mail_log), 0, 'No notification was sent')
        self.failUnlessEqual(len(mailhost.mail_log), 1, 
                             'More than one notification was sent')
        mail = mailhost.mail_log[0]
        self.failUnlessEqual(mail['to'], 'manager@bar.no', 
                             'Wrong person invited')
        self.failUnless(mail['subject'].find('modifi') != -1 )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(testNotification),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
