# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
# Author: Lennart Regebro <regebro@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: test_notification.py 26191 2005-08-29 10:12:53Z lregebro $

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest, transaction
from Testing.ZopeTestCase import user_name
from datetime import datetime, timedelta
from DateTime.DateTime import DateTime

from Products.CPSInstaller.CPSInstaller import CPSInstaller
try:
    from Products.CPSCalendar.Event import Event
except ImportError:
    Event = None

from calcore import cal, interfaces
from Products.CPSDefault.tests.CPSDefaultTestCase import CPSDefaultTestCase
from Products.ExternalMethod.ExternalMethod import ExternalMethod

manager_id = 'manager'
user_id = user_name

class TestUpgrade(CPSDefaultTestCase):

    def afterSetUp(self):
        self.login(manager_id)
        if 'install_cpscalendar' not in self.portal.objectIds():
            script = ExternalMethod('install_cpscalendar', '', 
                                    'CPSCalendar.install',
                                    'install')
            self.portal._setObject('install_cpscalendar', script)
            script = self.portal['install_cpscalendar']
            script()
        # The old versions of the CPSCalendar installer fails to do this,
        # so we do this all the time, to make sure it's done, even when you
        # try to run this on older CPSCalendar versions. After all, this 
        # should be able to upgrade older CPSCalendars, and then it's good to be
        # able to run the unit test with them:
        installer = CPSInstaller(self.portal, 'CPSCalendar')
        installer.verifyTool('portal_calendar', 'CMFCalendar', 'CMF Calendar Tool')
        installer.verifyTool('portal_cpscalendar', 'CPSCalendar', 'CPS Calendar Tool')
        self._createOldCalendars()
        transaction.commit()

    def _upgrade(self):
        if 'install_cpssharedcalendar' not in self.portal.objectIds():
            script = ExternalMethod('install_cpssharedcalendar', '', 
                                    'CPSSharedCalendar.install',
                                    'install')
            self.portal._setObject('install_cpssharedcalendar', script)
            script = self.portal['install_cpssharedcalendar']
            script()
        else:
            installer = CPSInstaller(self.portal, 'CPSCalendar')
            installer.verifyTool('portal_calendar', 'CPSSharedCalendar', 
                                 'CPS Calendar Tool')
            installer.verifyTool('portal_cpscalendar', 'CPSSharedCalendar', 
                                 'CPS Shared Calendar Tool')            
        transaction.commit()

        if 'migrate_cpscalendar' not in self.portal.objectIds():
            script = ExternalMethod('migrate_cpscalendar', '', 
                                    'CPSSharedCalendar.upgrade',
                                    'migrate_from_cpscalendar')
            self.portal._setObject('migrate_cpscalendar', script)
        script = self.portal['migrate_cpscalendar']
        script()
        #transaction.commit()

    def _createOldCalendars(self):
        # Set up the home folders and calendars:
        mtool = self.portal.portal_membership
        # Just delete all the old ones. Begone!
        for uid in self.portal.workspaces.members.objectIds():
            if uid[0] != '.':
                self.portal.workspaces.members._delObject(uid)
        for uid in (user_id, manager_id):
            mtool.createMemberArea(uid)

    def testCalendarUpgrade(self):
        self._upgrade()
        # Check that the calendars have been upgraded:
        cpstool = self.portal.portal_cpscalendar
        for uid in (user_id, manager_id):
            cal = cpstool.getHomeCalendarObject(uid)
            self.failUnlessEqual(cal.meta_type, 'CPS Calendar') 
        
    def testBasicEvents(self):
        # Create events:
        caltool = self.portal.portal_cpscalendar
        usercal = caltool.getHomeCalendarObject(user_id)
        
        # Create a bunch of events. 
        basedate =  DateTime(2005, 4, 1, 8, 0)
        event_count = 5

        for x in range(event_count):
            dtstart = basedate + float(x%7)/24
            duration = (1.0+x%3)/24
            uid = 'event' + str(x)
            event = Event(id=uid, title="This is event "+ uid, attendees=None, 
                          from_date=dtstart, to_date=dtstart+duration)
            usercal._setObject(uid, event)
            
            # Increase to next day
            if not x%3:
                basedate = basedate + 1

        # Upgrade the calendars
        self._upgrade()
                
        # There should still be the same number of events...
        sm = self.portal.portal_calendar.storage_manager
        events = sm.getEvents((None, None))
        self.failUnlessEqual(len(events), event_count)

        basedate = datetime(2005, 4, 1, 8, 0)
        for x in range(event_count):
            uid = 'event' + str(x)
            event = sm.getEvent(uid)
            self.failUnlessEqual(event.getOrganizerId(), 
                                 'INDIVIDUAL!' + user_id)
            self.failUnlessEqual(len(event.getAttendeeIds()), 1)

            dtstart = basedate + timedelta(hours=x%7)
            duration = timedelta(hours=1+x%3)
            self.failUnlessEqual(event.dtstart, dtstart)
            self.failUnlessEqual(event.duration, duration)
            self.failUnlessEqual(event.title, "This is event "+ uid)
            
            if not x%3:
                basedate = basedate + timedelta(1)
    
    def testAllDayEvents(self):
        # For the manager. We make these allday.
        caltool = self.portal.portal_cpscalendar
        mgrcal = caltool.getHomeCalendarObject(manager_id)
        # Create a bunch of all day events. 
        basedate =  DateTime(2005, 4, 1, 0, 0)
        event_count = 5
        for x in range(event_count):
            dtstart = basedate + (x%7)/24
            duration = (1+x%3)/24
            uid = 'event' + str(x)
            event = Event(id=uid, title="This is event "+ uid, 
                          attendees=None, from_date=dtstart, 
                          to_date=dtstart+duration, event_type='event_allday')
            mgrcal._setObject(uid, event)
            if not x%3:
                basedate = basedate + 1
        
        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        basedate = datetime(2005, 4, 1, 0, 0)
        for x in range(event_count):
            event = sm.getEvent('event' + str(x))
            self.failUnlessEqual(event.getOrganizerId(), 
                                 'INDIVIDUAL!' + manager_id)
            self.failUnlessEqual(len(event.getAttendeeIds()), 1)
            self.failUnlessEqual(event.dtstart, basedate)
            self.failUnlessEqual(event.duration, timedelta(1))
            self.failUnless(event.allday)
            
            if not x%3:
                basedate = basedate + timedelta(1)

    def testDailyRecurringEvents(self):
        caltool = self.portal.portal_cpscalendar
        mgrcal = caltool.getHomeCalendarObject(user_id)
        
        event = Event('daily', 
                      title="This is a daily event", 
                      attendees=None, 
                      from_date=DateTime(2005, 4, 1, 8, 0), 
                      to_date=DateTime(2005, 4, 1, 10, 0), 
                      event_type='event_recurring',
                      recurrence_period='period_daily')
        mgrcal._setObject('daily', event)

        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        event = sm.getEvent('daily')
        recurrence = event.recurrence
        self.failUnless(interfaces.IDailyRecurrenceRule.providedBy(recurrence))
        self.failUnlessEqual(recurrence.interval, 1)
        self.failUnlessEqual(recurrence.until, None)
            
    def testWeeklyRecurringEvents(self):
        caltool = self.portal.portal_cpscalendar
        mgrcal = caltool.getHomeCalendarObject(manager_id)
        
        event = Event('weekly', 
                      title="This is a daily event", 
                      attendees=None, 
                      from_date=DateTime(2005, 4, 1, 8, 0), 
                      to_date=DateTime(2005, 4, 1, 10, 0), 
                      event_type='event_recurring',
                      recurrence_period='period_weekly')
        mgrcal._setObject('weekly', event)

        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        event = sm.getEvent('weekly')
        recurrence = event.recurrence
        self.failUnless(interfaces.IWeeklyRecurrenceRule.providedBy(recurrence))
        self.failUnlessEqual(recurrence.interval, 1)
        self.failUnlessEqual(recurrence.until, None)

    def testMonthlyRecurringEvents(self):
        caltool = self.portal.portal_cpscalendar
        mgrcal = caltool.getHomeCalendarObject(user_id)
        
        event = Event('monthly', 
                      title="This is a monthly event", 
                      attendees=None, 
                      from_date=DateTime(2005, 4, 1, 8, 0), 
                      to_date=DateTime(2005, 4, 1, 10, 0), 
                      event_type='event_recurring',
                      recurrence_period='period_monthly')
        mgrcal._setObject('monthly', event)

        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        event = sm.getEvent('monthly')
        recurrence = event.recurrence
        self.failUnless(interfaces.IMonthlyRecurrenceRule.providedBy(recurrence))
        self.failUnlessEqual(recurrence.interval, 1)
        self.failUnlessEqual(recurrence.until, None)

    def testQuarterlyRecurringEvents(self):
        caltool = self.portal.portal_cpscalendar
        mgrcal = caltool.getHomeCalendarObject(manager_id)
        
        event = Event('quarterly', 
                      title="This is a quarterly event", 
                      attendees=None, 
                      from_date=DateTime(2005, 4, 1, 8, 0), 
                      to_date=DateTime(2005, 4, 1, 10, 0), 
                      event_type='event_recurring',
                      recurrence_period='period_quarterly')
        mgrcal._setObject('quarterly', event)

        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        event = sm.getEvent('quarterly')
        recurrence = event.recurrence
        self.failUnless(interfaces.IMonthlyRecurrenceRule.providedBy(recurrence))
        self.failUnlessEqual(recurrence.interval, 3)
        self.failUnlessEqual(recurrence.until, None)

    def testYearlyRecurringEvents(self):
        caltool = self.portal.portal_cpscalendar
        mgrcal = caltool.getHomeCalendarObject(manager_id)
        
        event = Event('yearly', 
                      title="This is a yearly event", 
                      attendees=None, 
                      from_date=DateTime(2005, 4, 1, 8, 0), 
                      to_date=DateTime(2005, 4, 1, 10, 0), 
                      event_type='event_recurring',
                      recurrence_period='period_yearly')
        mgrcal._setObject('yearly', event)

        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        event = sm.getEvent('yearly')
        recurrence = event.recurrence
        self.failUnless(interfaces.IYearlyRecurrenceRule.providedBy(recurrence))
        self.failUnlessEqual(recurrence.interval, 1)
        self.failUnlessEqual(recurrence.until, None)

    def testMeetings(self):
        ctool = self.portal.portal_cpscalendar
        mgrcal = ctool.getHomeCalendarObject(manager_id)
        usrcal = ctool.getHomeCalendarObject(user_id)
        atts = [ctool.getAttendeeInfo(cal.getRpath(), 1) for cal in (mgrcal, usrcal)]
        org = atts[1].copy()
        del org['status']
        atts[0]['status'] = 'confirmed'
        event = Event('meeting1', 
                      title="One meeting", 
                      organizer=org,
                      attendees=atts,
                      from_date=DateTime(2005, 4, 1, 8, 0), 
                      to_date=DateTime(2005, 4, 1, 10, 0))
        mgrcal._setObject('meeting1', event)

        event = mgrcal['meeting1']
        event.updateAttendeesCalendars()

        org = atts[0].copy()
        del org['status']
        atts[1]['status'] = 'decline'
        event = Event('meeting2', 
                      title="Two meeting", 
                      organizer=org,
                      attendees=atts,
                      from_date=DateTime(2005, 4, 2, 8, 0), 
                      to_date=DateTime(2005, 4, 2, 10, 0))
        mgrcal._setObject('meeting2', event)

        event = mgrcal['meeting2']
        event.updateAttendeesCalendars()

        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        asrc = self.portal.portal_calendar.attendee_source
        useratt = asrc.getAttendee('INDIVIDUAL!'+user_id)
        mgratt = asrc.getAttendee('INDIVIDUAL!'+manager_id)
        
        event = sm.getEvent('meeting1')
        self.failUnlessEqual(len(event.getAttendeeIds()), 2)
        self.failUnlessEqual(event.getParticipationStatus(mgratt), 'ACCEPTED')
        self.failUnlessEqual(event.getParticipationStatus(useratt), 
                             'NEEDS-ACTION')
        self.failUnlessEqual(event.getOrganizerId(), 'INDIVIDUAL!'+user_id)

        event = sm.getEvent('meeting2')
        self.failUnlessEqual(len(event.getAttendeeIds()), 2)
        self.failUnlessEqual(event.getParticipationStatus(mgratt), 'ACCEPTED')
        self.failUnlessEqual(event.getParticipationStatus(useratt), 'DECLINED')
        self.failUnlessEqual(event.getOrganizerId(), 'INDIVIDUAL!'+manager_id)


    def testAllFields(self):
        # Make sure all fields are converted
        ctool = self.portal.portal_cpscalendar
        cal = ctool.getHomeCalendarObject(user_id)
        event = Event('event', 
                      title='Title', 
                      attendees=None,
                      from_date=DateTime(2005, 4, 1, 8, 0), 
                      to_date=DateTime(2005, 4, 1, 10, 0), 
                      event_type='event_tofrom',
                      description='Description',
                      location='Location',
                      status='canceled',
                      document_link='document.doc',
                      transparent=1,
                      category='private',
                      document='workspaces/a_document_id',
                      )
        cal._setObject('event', event)

        self._upgrade()

        sm = self.portal.portal_calendar.storage_manager
        event = sm.getEvent('event')
        self.failUnlessEqual(event.title, 'Title')
        self.failUnlessEqual(event.description, 'Description')
        self.failUnlessEqual(event.location, 'Location')
        self.failUnlessEqual(event.status, 'CANCELLED')
        self.failUnlessEqual(event.transparent, True)
        self.failUnlessEqual(event.access, 'PRIVATE')
        self.failUnlessEqual(event.document, 'workspaces/a_document_id')

def test_suite():
    #return None # temp fix
    if Event is None:
        # No CPSCalendar could be imported, no upgrade possible, test can not run.
        return None
    return unittest.TestSuite((
        unittest.makeSuite(TestUpgrade),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
