# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: interfaces.py 24640 2005-07-04 17:02:11Z lregebro $
"""
  interfaces.py
"""
from zope.interface import Interface
from zope.schema import Int, Date, Datetime, TextLine, Text, Bool, Choice, Set

class IUnPositionedView(Interface):
    """A view that uses no positioning to render events (month view)"""
    pass

class IPositionedView(Interface):
    """A view that uses absolute positioning to render events"""
    
    width = Int(title=u"Width", 
                description=u"The total width of the view")
    
    height = Int(title=u"Height", 
                description=u"The total height of the view")
    
    first_day = Date(title=u"First Day",
                     description=u"The first day of the view")
    
    days = Int(title=u"Days", 
               description=u"The number of days shown")

    from_hour = Int(title=u"From hour", 
                    description=u"The time at the top of the view")

    to_hour = Int(title=u"To hour", 
                  description=u"The time at the bottom of the view")


class IEventPosition(Interface):
    """An events position

    Contains the information needed to properly display an event on a 
    IPositionedView."""
    
    def getCssPositionString():
        """Returns a properly formatted css-style position string.

        Usage in the template:
          tal:attributes=u"style event_render_info/getCssPositionString"
        """

class IEventDisplay(Interface):
    """An events display information

    Contains the information needed to properly display an event on a 
    IPositionedView."""
    
    def getCssPositionString():
        """Returns a properly formatted css-style position string.

        Usage in the template:
          tal:attributes=u"style event_render_info/getCssPositionString"
        """

    def getCssStatusClass(self):
        """Returns a CSS class name according to the status of this event
        for the current user attendee (if there is one). Possible classes are:
            
            need-action
            accepted
            declined
            tentative
            delegated
            <empty>
            
        It is up to the actual stylesheet to do something with this class.
        """

class IEventEditSchema(Interface):
    title = TextLine(
        title=u"Title",
        description=u"The title of the event.")

    description = Text(
        title=u"Description",
        required=False,
        description=u"A description of the event.")

    dtstart = Datetime(title=u'Starts',
                       description=u"Date and time when this event starts.")

    dtend = Datetime(title=u'Ends',
                     description=u"Date and time when this event ends.")

    allday = Bool(
        title=u"Takes all day",
        description=u"Check this box if the event goes on all day",
        required=False)

    transparent = Bool(
        title=u"Not busy",
        description=u"This event should not appear as busy time in the meeting helper.",
        required=False)
          
    location = Text(
        title=u"Location",
        required=False,
        description=u"The location where this event takes place.")
    
    status = Choice(
        title=u"Status",
        values=('TENTATIVE', 'CONFIRMED', 'CANCELED'),
        description=u"The status of the event.",
        required=True,
        default='CONFIRMED')

    access = Choice(
        title=u"Access",
        values=('PUBLIC', 'PRIVATE',),
        description=u"Who has access to this event.",
        required=True,
        default='PUBLIC')

    document = TextLine(
        title=u"Document",
        required=False,
        description=u"A document related to this event.")
    
    categories = Set(
        title=u"Categories",
        description=u"Categories for this event",
        value_type=Text(),
        required=False)
