# -*- coding: ISO-8859-15 -*-
# (C) Copyright 2005 Nuxeo SARL <http://nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: zopecal.py 27771 2005-09-30 13:41:37Z lregebro $

from calcore import cal, isoweek

# python
from random import randrange
from datetime import datetime, timedelta, date
import calendar

# zope
from OFS.SimpleItem import SimpleItem
from OFS.Folder import Folder
from ZODB.PersistentMapping import PersistentMapping
from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

import helpers
import storage
from interfaces import IYear, IMonth, IDay
from interfaces import IEventList, IWeekList, IWeekYear, IWeek
from zope.interface import implements
from zope.app.container.interfaces import IContainerNamesContainer
from Products.Five.traversable import  FiveTraversable

from calcore.interfaces import IAttendeeSource

# XXX security implications of inherited python code!
# Yeah, what about them? //Regebro

class CalendarTool(Folder):
    
    meta_type = 'Calendar Tool'
        
InitializeClass(CalendarTool)

def manage_addCalendarTool(context, REQUEST=None):
    """Add a calendar tool"""
    id = 'portal_calendar'
    obj = CalendarTool(id)
    destination = getattr(context, 'Destination', None)
    if destination:
        context = destination()
    context._setObject(id, obj)
    obj = context[id]
    manage_addStorageManager(obj, 'storage_manager', '')
    manage_addUserFolderAttendeeSource(obj, 'attendee_source', '')
    helpers.add_and_edit(context, id, REQUEST)
    return ''

class StorageManager(SimpleItem, cal.StorageManagerBase):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Storage Manager'
    
    def __init__(self, id, title):
        StorageManager.inheritedAttribute('__init__')(self)
        self.id = id
        self.title = title
        # simple ZODB storage hardcoded in for now
        self.setStorage(storage.ZODBStorage('storage'))

    def notifyOnStatusChange(self, attendee, event, old_status, new_status):
        self._storage.reindexEvent(event.unique_id, event)

    def notifyOnEventModification(self, event):
        self._storage.reindexEvent(event.unique_id, event)
                                  
    def createEvent(self, unique_id=None, **kw):
        return self._storage.createEvent(unique_id, 
                                         storage.ZopeEventSpecification(**kw))

InitializeClass(StorageManager)

manage_addStorageManagerForm = PageTemplateFile(
    "www/storageManagerAdd", globals(),
    __name__='manage_addStorageManagerForm')

def manage_addStorageManager(context, id, title, REQUEST=None):
    """Add a calendar storage manager"""
    obj = StorageManager(id, title)
    context._setObject(id, obj)
    helpers.add_and_edit(context, id, REQUEST)
    return ''

class UserFolderAttendeeSource(SimpleItem):
    """A very simplistic attendee source, designed to be replaced.
    """
    security = ClassSecurityInfo()

    meta_type = 'Calendar Attendee Source (User Folder)'

    implements(IAttendeeSource)
    
    def __init__(self, id, title):
        self.id = id
        self.title = title

    def getMainCalendarForAttendeeId(self, attendee_id):
        # This should be overridden in more advance portal implementations.
        # This implementation knows nothing about home calendars for 
        # individuals and it only knows about individuals. So there ya go.
        return None
        
    def getAttendeeTypes(self):
        return ['INDIVIDUAL']

    def getAttendee(self, attendee_id):
        # XXX no checking yet whether user exists at all
        attendee = Attendee(attendee_id, attendee_id, 'INDIVIDUAL')
        homecal = self.getMainCalendarForAttendeeId(attendee_id)
        if homecal is None:
            return attendee.__of__(self)
        # This is to acquire the security settings from the calendar:
        return attendee.__of__(homecal)

    def getCurrentUserAttendee(self):
        return self.getAttendee(getSecurityManager().getUser().getId())
        
    def findByName(self, query_str, attendee_type=None):
        # don't deal with anything but individuals
        if attendee_type is not None and attendee_type != 'INDIVIDUAL':
            return []
        result = []
        for userid in self.get_valid_userids():
            if query_str not in userid:
                continue
            result.append(self.getAttendee(userid))
        return result
    
    def getAttendeesOfType(self, attendee_type):
        result = []
        if attendee_type != 'INDIVIDUAL':
            return []
        for userid in self.get_valid_userids():
            result.append(self.getAttendee(userid))
        return result

    def getAttendeeFromSpec(self, vcaladdress):
        id = vcaladdress.decode()
        return self._attendees.get(id, None)

InitializeClass(UserFolderAttendeeSource)

manage_addUserFolderAttendeeSourceForm = PageTemplateFile(
    "www/ufAttendeeSourceAdd", globals(),
    __name__='manage_addUserFolderAttendeeSourceForm')

def manage_addUserFolderAttendeeSource(context, id, title, REQUEST=None):
    """Add a user-folder based attendee source"""
    obj = UserFolderAttendeeSource(id, title)
    context._setObject(id, obj)
    helpers.add_and_edit(context, id, REQUEST)
    return ''

class Attendee(SimpleItem, cal.AttendeeBase):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Attendee'
    
    def __init__(self, attendee_id, name, attendee_type, on_invite=None):
        Attendee.inheritedAttribute('__init__')(
            self, attendee_id, name, attendee_type, on_invite)
        self.id = attendee_id
        self.title = name
    
    def _getStorageManager(self):
        return self.portal_calendar.storage_manager

    def on_status_change(self, event, old_status, new_status):
        # This is called also on invites, so overriding on_invite
        # is not needed.
        self._getStorageManager().notifyOnStatusChange(
            self, event, old_status, new_status)

    def getUserId(self):
        return self.id

InitializeClass(Attendee)

class Calendar(SimpleItem, cal.CalendarBase):
    security = ClassSecurityInfo()
    implements(IContainerNamesContainer)

    meta_type = 'Calendar'
    
    def __init__(self, id, title='', description=''):
        Calendar.inheritedAttribute('__init__')(self)
        self.id = id
        self.title = title
        self.description = description

    def _initAttendees(self):
        return PersistentMapping()

    # purely used for acquisition
    def getCalendar(self):
        return self.aq_inner

    # getting attendees in right acquisition context, is needed
    # as attendee tries to do _getCalendar which is implemented using
    # acquisition
    def getAttendees(self):
        result = []
        for attendee in Calendar.inheritedAttribute('getAttendees')(self):
            result.append(attendee.__of__(self))
        return result

    def getWeekNr(self):
        return datetime.today().isocalendar()[1]

    def getYear(self):
        return datetime.today().year
    
    def _getStorageManager(self):
        return self.portal_calendar.storage_manager

    def _getAttendeeSource(self):
        return self.portal_calendar.attendee_source
    
class CalendarTraversable(FiveTraversable):

    # url space

    # day: calendar/2005/05/30
    # month: calendar/2005/05
    # year: calendar/2005
    # week: calendar/week/2005/3
    # weekday: calendar/week/2005/3/2
    # event: calendar/event/1223

    def traverse(self, name, furtherPath):
        if name == 'event':
            return EventList().__of__(self._subject)
        elif name == 'week':
            return WeekList().__of__(self._subject)
        else:
            try:
                year = int(name)
                return Year(year).__of__(self._subject)
            except ValueError:
                pass
        if name == 'getId':
            # XXX Special hack I don't understand.
            # Maybe I can get rid of it now when I use Traversal adapters?
            return getattr(self._subject, name)
        
        return FiveTraversable.traverse(self, name, furtherPath)


InitializeClass(Calendar)

manage_addCalendarForm = PageTemplateFile(
    "www/calendarAdd", globals(),
    __name__='manage_addCalendarForm')

def manage_addCalendar(context, id, title, REQUEST=None):
    """Add a calendar."""
    obj = Calendar(id, title)
    context._setObject(id, obj)
    helpers.add_and_edit(context, id, REQUEST)
    return ''

class Year(SimpleItem):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Year'

    implements(IYear)
    
    def __init__(self, year):
        self.id = str(year)
        self.title = self.id
        self._year = year

    def getYear(self):
        return self._year

    def getMonths(self):
        return range(1, 13)

class YearTraversable(FiveTraversable):
    
    def traverse(self, name, furtherPath):
        try:
            month = int(name)
            return Month(self._subject._year, month).__of__(self._subject)
        except ValueError:
            pass
        return FiveTraversable.traverse(self, name, furtherPath)

            
InitializeClass(Year)

class Month(SimpleItem):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Month'

    implements(IMonth, IContainerNamesContainer)
    
    def __init__(self, year, month):
        self.id = str(month)
        self.title = self.id
        self._year = year
        self._month = month

    def getYear(self):
        return self._year

    def getMonth(self):
        return self._month

    def getDays(self):
        weekday, amount_days = calendar.monthrange(self._year, self._month)
        return range(1, amount_days + 1)

    def getDateForDay(self, day):
        return datetime(self._year, self._month, day)

class MonthTraversable(FiveTraversable):

    def traverse(self, name, furtherPath):
        try:
            day = int(name)
            return Day(self._subject._year, self._subject._month, day).__of__(self._subject)
        except ValueError:
            pass
        return FiveTraversable.traverse(self, name, furtherPath)

InitializeClass(Month)

class Day(SimpleItem):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Day'

    implements(IDay, IContainerNamesContainer)
    
    def __init__(self, year, month, day):
        self.id = str(day)
        self.title = self.id
        self._year = year
        self._month = month
        self._day = day

    def getYear(self):
        return self._year

    def getMonth(self):
        return self._month

    def getDay(self):
        return self._day
    
InitializeClass(Day)

class EventList(SimpleItem):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Event List'

    implements(IEventList)
    
    def __init__(self):
        self.id = 'event'
        self.title = self.id

class EventListTraversable(FiveTraversable):

    def traverse(self, name, furtherPath):
        try:
            calendar = self._subject.getCalendar()
            event = calendar.getCalendar().getEvent(name)
            c_attendees = calendar.getAttendees()
            e_attendees = event.getAttendeeIds()
            for attendee in c_attendees:
                if attendee.getAttendeeId() in e_attendees:
                    return event.__of__(self._subject)
            # None of this calendars attendees are attendees on the event.
            # The event should not be displayed:
            raise AttributeError
        except (ValueError, KeyError):
            pass
        return FiveTraversable.traverse(self, name, furtherPath)

InitializeClass(EventList)

class WeekList(SimpleItem):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Week List'

    implements(IWeekList)
    
    def __init__(self):
        self.id = 'week'
        self.title = self.id

class WeekListTraversable(FiveTraversable):

    def traverse(self, name, furtherPath):
        try:
            year = int(name)
            return WeekYear(year).__of__(self._subject)
        except ValueError:
            pass
        return FiveTraversable.traverse(self, name, furtherPath)

InitializeClass(WeekList)

class WeekYear(SimpleItem):
    meta_type = 'Calendar Week Year'

    implements(IWeekYear)

    def __init__(self, year):
        self.id = str(year)
        self.title = self.id
        self._year = year

    def getYear(self):
        return self._year

    def getWeeks(self):
        return range(1, isoweek.getWeeksInYear(self._year) + 1)
    
class WeekYearTraversable(FiveTraversable):

    def traverse(self, name, furtherPath):
        try:
            week_nr = int(name)
            return Week(self._subject._year, week_nr).__of__(self._subject)
        except ValueError:
            pass
        return FiveTraversable.traverse(self, name, furtherPath)

InitializeClass(WeekYear)

class Week(SimpleItem):
    security = ClassSecurityInfo()

    meta_type = 'Calendar Week'

    implements(IWeek, IContainerNamesContainer)
    
    def __init__(self, year, week_nr):
        self.id = str(week_nr)
        self.title = self.id
        self._year = year
        self._week_nr = week_nr

    def getYear(self):
        return self._year

    def getWeekNr(self):
        return self._week_nr

##     def getWeekdays(self):
##         return range(1, 8)

    def getDateForWeekday(self, weekday):
        return isoweek.weeknr2datetime(self._year, self._week_nr, weekday)

class WeekTraversable(FiveTraversable):

    def traverse(self, name, furtherPath):
        try:
            d = self._subject.getDateForWeekday(int(name))
            return Day(d.year, d.month, d.day).__of__(self._subject)
        except ValueError:
            pass
        return FiveTraversable.traverse(self, name, furtherPath)
    
InitializeClass(Week)
